@@ -1,12 +1,18 @@
-/* $Id: daemon.c 4022 2008-03-31 06:11:07Z rra $
- *
+/*
  * Replacement for a missing daemon.
  *
  * Provides the same functionality as the library function daemon for those
  * systems that don't have it.
  *
  * Written by Russ Allbery <rra@stanford.edu>
- * This work is hereby placed in the public domain by its author.
+ *
+ * The authors hereby relinquish any claim to any copyright that they may have
+ * in this work, whether granted under contract or by operation of law or
+ * international treaty, and hereby commit to the public, at large, that they
+ * shall not, at any time in the future, seek to enforce any copyright in this
+ * work against any person or entity, or prevent any person or entity from
+ * copying, publishing, distributing or creating derivative works of this
+ * work.
  */
 
 #include <config.h>
@@ -17,6 +23,17 @@
 #include <sys/ioctl.h>
 #include <sys/stat.h>
 
+/*
+ * If we're running the test suite, rename daemon to avoid conflicts with the
+ * system version.  #undef it first because some systems may define it to
+ * another name.
+ */
+#if TESTING
+# undef daemon
+# define daemon test_daemon
+int test_daemon(int, int);
+#endif
+
 int
 daemon(int nochdir, int noclose)
 {
