@@ -10,7 +10,14 @@
  * than directly.
  *
  * Written by Russ Allbery <rra@stanford.edu>
- * This work is hereby placed in the public domain by its author.
+ *
+ * The authors hereby relinquish any claim to any copyright that they may have
+ * in this work, whether granted under contract or by operation of law or
+ * international treaty, and hereby commit to the public, at large, that they
+ * shall not, at any time in the future, seek to enforce any copyright in this
+ * work against any person or entity, or prevent any person or entity from
+ * copying, publishing, distributing or creating derivative works of this
+ * work.
  */
 
 #ifndef PORTABLE_GETADDRINFO_H
@@ -22,6 +29,8 @@
 /* Skip this entire file if a system getaddrinfo was detected. */
 #ifndef HAVE_GETADDRINFO
 
+/* OpenBSD likes to have sys/types.h included before sys/socket.h. */
+#include <sys/types.h>
 #include <sys/socket.h>
 
 /* The struct returned by getaddrinfo, from RFC 3493. */
