@@ -1,13 +1,19 @@
-/* $Id: stdbool.h 4017 2008-03-30 23:14:04Z rra $
- *
+/*
  * Portability wrapper around <stdbool.h>.
  *
- * Written by Russ Allbery <rra@stanford.edu>
- * This work is hereby placed in the public domain by its author.
- *
  * Provides the bool and _Bool types and the true and false constants,
  * following the C99 specification, on hosts that don't have stdbool.h.  This
  * logic is based heavily on the example in the Autoconf manual.
+ *
+ * Written by Russ Allbery <rra@stanford.edu>
+ *
+ * The authors hereby relinquish any claim to any copyright that they may have
+ * in this work, whether granted under contract or by operation of law or
+ * international treaty, and hereby commit to the public, at large, that they
+ * shall not, at any time in the future, seek to enforce any copyright in this
+ * work against any person or entity, or prevent any person or entity from
+ * copying, publishing, distributing or creating derivative works of this
+ * work.
  */
 
 #ifndef PORTABLE_STDBOOL_H
@@ -16,7 +22,9 @@
 #if HAVE_STDBOOL_H
 # include <stdbool.h>
 #else
-# if !HAVE__BOOL
+# if HAVE__BOOL
+#  define bool _Bool
+# else
 #  ifdef __cplusplus
 typedef bool _Bool;
 #  elif _WIN32
