@@ -7,7 +7,8 @@
  *
  * Original implementation by Thomas L. Kula <kula@tproa.net>
  * Copyright 2008 Thomas L. Kula <kula@tproa.net>
- * Copyright 2008 Board of Trustees, Leland Stanford Jr. University
+ * Copyright 2008
+ *     The Board of Trustees of the Leland Stanford Junior University
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -99,7 +100,7 @@ py_remctl(PyObject *self, PyObject *args)
     }
     result = Py_BuildValue("(ss#s#i)", rr->error,
                            rr->stdout_buf, (int) rr->stdout_len,
-                           rr->stderr_buf, (int) rr->stderr_buf,
+                           rr->stderr_buf, (int) rr->stderr_len,
                            rr->status);
     remctl_result_free(rr);
 
