@@ -2,10 +2,14 @@
  * asprintf and vasprintf test suite.
  *
  * Written by Russ Allbery <rra@stanford.edu>
- * Copyright 2006, 2008, 2009
- *     Board of Trustees, Leland Stanford Jr. University
  *
- * See LICENSE for licensing terms.
+ * The authors hereby relinquish any claim to any copyright that they may have
+ * in this work, whether granted under contract or by operation of law or
+ * international treaty, and hereby commit to the public, at large, that they
+ * shall not, at any time in the future, seek to enforce any copyright in this
+ * work against any person or entity, or prevent any person or entity from
+ * copying, publishing, distributing or creating derivative works of this
+ * work.
  */
 
 #include <config.h>
@@ -13,7 +17,8 @@
 
 #include <tests/tap/basic.h>
 
-int test_asprintf(char **, const char *, ...);
+int test_asprintf(char **, const char *, ...)
+    __attribute__((__format__(printf, 2, 3)));
 int test_vasprintf(char **, const char *, va_list);
 
 static int
