@@ -2,13 +2,14 @@
  * getaddrinfo test suite.
  *
  * Written by Russ Allbery <rra@stanford.edu>
- * Copyright 2009 Board of Trustees, Leland Stanford Jr. University
- * Copyright (c) 2004, 2005, 2006, 2007
- *     by Internet Systems Consortium, Inc. ("ISC")
- * Copyright (c) 1991, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
- *     2002, 2003 by The Internet Software Consortium and Rich Salz
  *
- * See LICENSE for licensing terms.
+ * The authors hereby relinquish any claim to any copyright that they may have
+ * in this work, whether granted under contract or by operation of law or
+ * international treaty, and hereby commit to the public, at large, that they
+ * shall not, at any time in the future, seek to enforce any copyright in this
+ * work against any person or entity, or prevent any person or entity from
+ * copying, publishing, distributing or creating derivative works of this
+ * work.
  */
 
 #include <config.h>
@@ -45,7 +46,7 @@ main(void)
     struct hostent *host;
     struct in_addr addr;
     struct servent *service;
-    int i;
+    int i, result;
     int found;
 
     plan(75);
@@ -227,16 +228,21 @@ main(void)
         test_freeaddrinfo(ai);
     }
 
-    hints.ai_socktype = SOCK_STREAM;
-    hints.ai_flags = AI_CANONNAME;
-    ok(test_getaddrinfo("foo.invalid", NULL, NULL, &ai) == EAI_NONAME,
-       "lookup of invalid address");
+    host = gethostbyname("addrinfo-test.invalid");
+    if (host != NULL)
+        skip("lookup of addrinfo-test.invalid succeeded");
+    else {
+        result = test_getaddrinfo("addrinfo-test.invalid", NULL, NULL, &ai);
+        is_int(EAI_NONAME, result, "lookup of invalid address");
+    }
 
     host = gethostbyname("cnn.com");
     if (host == NULL) {
         skip_block(3, "cannot look up cnn.com");
         exit(0);
     }
+    hints.ai_socktype = SOCK_STREAM;
+    hints.ai_flags = AI_CANONNAME;
     ok(test_getaddrinfo("cnn.com", NULL, &hints, &ai) == 0,
        "lookup of cnn.com");
     saddr = (struct sockaddr_in *) ai->ai_addr;
