@@ -1,13 +1,27 @@
 /*
  * Test suite for xmalloc and family.
  *
- * Copyright 2008 Board of Trustees, Leland Stanford Jr. University
- * Copyright 2004, 2005, 2006
- *     by Internet Systems Consortium, Inc. ("ISC")
- * Copyright 1991, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
- *     2003 by The Internet Software Consortium and Rich Salz
+ * Copyright 2000, 2001, 2006 Russ Allbery <rra@stanford.edu>
+ * Copyright 2008
+ *     The Board of Trustees of the Leland Stanford Junior University
  *
- * See LICENSE for licensing terms.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #line 1 "xmalloc.c"
@@ -185,6 +199,8 @@ test_asprintf(size_t size)
     string[size - 1] = '\0';
     status = xasprintf(&copy, "%s", string);
     free(string);
+    if (status < 0)
+        return 0;
     for (i = 0; i < size - 1; i++)
         if (copy[i] != 42)
             return 0;
@@ -225,6 +241,8 @@ test_vasprintf(size_t size)
     string[size - 1] = '\0';
     status = xvasprintf_wrapper(&copy, "%s", string);
     free(string);
+    if (status < 0)
+        return 0;
     for (i = 0; i < size - 1; i++)
         if (copy[i] != 42)
             return 0;
@@ -246,8 +264,6 @@ main(int argc, char *argv[])
     size_t limit = 0;
     int willfail = 0;
     unsigned char code;
-    struct rlimit rl;
-    void *tmp;
 
     if (argc < 3)
         die("Usage error.  Type, size, and limit must be given.");
@@ -290,6 +306,9 @@ main(int argc, char *argv[])
      */
     if (limit > 0) {
 #if HAVE_SETRLIMIT && defined(RLIMIT_AS)
+        struct rlimit rl;
+        void *tmp;
+
         rl.rlim_cur = limit;
         rl.rlim_max = limit;
         if (setrlimit(RLIMIT_AS, &rl) < 0) {
